/**
 * 
 */
package com.platomix.datasafe.service;

import java.io.File;
import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import com.platomix.datasafe.enums.TaskStateEnum;
import com.platomix.datasafe.model.EventDecode;
import com.platomix.datasafe.model.TaskInfo;
import com.platomix.datasafe.support.Consts;
import com.platomix.datasafe.support.FileKit;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 * 解密文件
 */
@Slf4j
@Component
public class EventDecodeListener implements ApplicationListener<EventDecode> {

	@Autowired
	private TaskInfoService taskInfoService;

	@Autowired
	private RsaKeyService rsaKeyService;

	@Async
	@Override
	public void onApplicationEvent(EventDecode event) {
		Long taskId = event.getTaskId();
		String key = rsaKeyService.getKey();
		String srcFilePath = event.getSrcFilePath();
		String destFilePath = Consts.getDownloadDir() + UUID.fastUUID().toString(true) + ".txt";
		File destFile = new File(destFilePath);
		log.info("开始解密手机号文件|taskId={},srcFilePath={}", taskId, srcFilePath);
		boolean ret = FileKit.decodeFile(srcFilePath, destFilePath, key);
		if(ret) {
			// 操作完成，更新taskId
			TaskInfo entity = new TaskInfo();
			entity.setTaskId(taskId);
			entity.setDestFileSize(destFile.length());
			entity.setFinishTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
			entity.setState(TaskStateEnum.SUCCESS);
			entity.setDestFilePath(destFile.getAbsolutePath());
			taskInfoService.updateById(entity);
		}
	}
}
