/**
 * 
 */
package com.platomix.datasafe.service;

import java.time.LocalDateTime;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.platomix.datasafe.enums.TaskStateEnum;
import com.platomix.datasafe.mapper.TaskInfoMapper;
import com.platomix.datasafe.model.EventDecode;
import com.platomix.datasafe.model.TaskInfo;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Service
public class TaskInfoService extends ServiceImpl<TaskInfoMapper, TaskInfo> {

	@Autowired
	private ApplicationContext applicationContext;

	/**
	 * 定时检测漏掉的未处理或者处理失败的重试1次
	 * 每隔5分钟检测一次，发现超过1小时未处理任务，继续处理
	 */
//	@Scheduled(initialDelay = 1000, fixedRate = 1 * 60 * 60 * 1000)
//	@Scheduled(cron = "0 0/5 * * * ?")
	public void checkMiss() {
		log.info("<<<<<<<<< check miss task start >>>>>>>>");
		String oneHourAgo = DateUtil.format(LocalDateTime.now().minusHours(1), "yyyy-MM-dd HH:mm:ss");
		LambdaQueryWrapper<TaskInfo> queryWrapper = Wrappers.<TaskInfo>lambdaQuery()
				.eq(TaskInfo::getState, TaskStateEnum.DOING).lt(TaskInfo::getCreateTime, oneHourAgo);
		List<TaskInfo> dataList = baseMapper.selectList(queryWrapper);
		for (TaskInfo taskInfo : dataList) {
			log.info("检测到一个未处理的任务|taskId={},createTime={}", taskInfo.getTaskId(),
					taskInfo.getCreateTime());
			applicationContext.publishEvent(new EventDecode(this, taskInfo.getTaskId(), taskInfo.getSrcFilePath()));
		}
	}
}
