/**
 * 
 */
package com.platomix.datasafe.support;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
public final class FileKit {
	/**
	 * 
	 * @param srcFilePath
	 * @param destFilePath
	 */
	public static boolean decodeFile(String srcFilePath, String destFilePath, String key) {
		return decodeFile(new File(srcFilePath), new File(destFilePath), key);
	}
	
	/**
	 * 
	 * @param srcFile
	 * @param destFile
	 * @param key
	 * @return
	 */
	public static boolean decodeFile(File srcFile, File destFile, String key) {
		long startTime = System.currentTimeMillis();
		// 缓冲大小设置4M
		try (BufferedReader bufferReader = new BufferedReader(new FileReader(srcFile), 2 << 12);
				BufferedWriter bufferWriter = new BufferedWriter(new FileWriter(destFile, true), 2 << 11);) {
			String line = null;
			List<String> mobileList = new ArrayList<String>();
			boolean firstInsert = true;
			while ((line = bufferReader.readLine()) != null) {
				// 解密
				String plaintext = RsaUtil.decrypt(key, line);
				if (plaintext != null) {
					mobileList.add(plaintext);
				}
				// 当mobileList条数等于500时，乱序后写入
				if (mobileList.size() == 1000) {
					_shuffleWrite(bufferWriter, mobileList, firstInsert);
					firstInsert = false;
					mobileList.clear();
				}
			}
			if (!mobileList.isEmpty()) {
				_shuffleWrite(bufferWriter, mobileList, firstInsert);
			}
			log.info("手机号解密完成，并且写入文件成功|cost={}ms,destFilePath={}", System.currentTimeMillis() - startTime,
					destFile.getAbsoluteFile());
			return true;
		} catch (Exception e) {
			log.error("解密手机号文件失败|errmsg=" + e.getMessage(), e);
			return false;
		}
	}
	
	/**
	 * 乱序插入
	 * @param bufferWriter
	 * @param mobileList
	 * @param firstInsert
	 * @throws IOException
	 */
	private static void _shuffleWrite(BufferedWriter bufferWriter, List<String> mobileList, boolean firstInsert) throws IOException {
		Collections.shuffle(mobileList);
		for(String item : mobileList) {
			if(!firstInsert) {
				bufferWriter.newLine();
			}
			bufferWriter.write(item);
			firstInsert = false;
		}
	}
}
