/**
 * 
 */
package com.platomix.datasafe.support;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.HandlerInterceptor;

import com.alibaba.fastjson.JSON;
import com.platomix.datasafe.enums.WhetherEnum;
import com.platomix.datasafe.model.Channel;
import com.platomix.datasafe.model.RequestLog;
import com.platomix.datasafe.service.ChannelService;
import com.platomix.datasafe.service.RequestLogService;

import cn.hutool.extra.servlet.ServletUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn 
 *  接口请求拦截器
 */
@Slf4j
public class RequestInterceptor implements HandlerInterceptor {

	@Autowired
	private RequestLogService requestLogService;
	
	@Autowired
	private ChannelService channelService;
	
	/**
	 *
	 */
	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
			throws Exception {
		//记录请求日志
		String uri = request.getRequestURI();
		String clientIp = ServletUtil.getClientIP(request);
		RequestLog requestLog = new RequestLog("SUCCESS", uri, clientIp);
		do {
			//获取文件大小或者记录数
			requestLog.setDataSize(request.getContentLengthLong());
			//获取调用渠道
			String channelCode = request.getParameter(Consts.CHANNEL_PARAM_NAME);
			//校验渠道来源
			if(channelCode == null || "".equals(channelCode)) {
				requestLog.setErrMsg("渠道编码不能为空"); requestLog.setState(WhetherEnum.NO); break;
			}
			Channel channel = channelService.getById(channelCode);
			if(channel == null) {
				requestLog.setErrMsg("渠道编码不正确,channel=" + channelCode); requestLog.setState(WhetherEnum.NO); break;
			}
			requestLog.setChannel(channel.getName());
			//获取调用的时间戳
//			Long timeStamp = Consts.toLongValue(request.getParameter(Consts.TIMESTAMP_PARAM_NAME), 0L);
//			if(timeStamp <= 0L) {
//				requestLog.setErrMsg("时间戳不能为空"); requestLog.setState(WhetherEnum.NO); break;
//			}
//			if(Math.abs(System.currentTimeMillis() - timeStamp) > 60000) {
//				requestLog.setErrMsg("调用时间必须在1分钟之内"); requestLog.setState(WhetherEnum.NO); break;
//			}
//			requestLog.setRequestTime(DateUtil.format(new Date(timeStamp), "yyyy-MM-dd HH:mm:ss"));
		}while(false);
		//入库请求日志
		requestLogService.save(requestLog);
		return requestLog.getState() == WhetherEnum.YES ? true : renderJson(response, JSON.toJSONString(R.fail(requestLog.getErrMsg())));
	}

	/**
	 * 
	 * @param response
	 * @param result
	 * @return
	 * @throws Exception
	 */
	private boolean renderJson(HttpServletResponse response, String result) throws Exception {
		response.setCharacterEncoding("UTF-8");
		response.setContentType("application/json; charset=utf-8");
		try (PrintWriter writer = response.getWriter()) {
			writer.print(result);
		} catch (IOException e) {
			log.error(e.getMessage(), e);
		}
		return false;
	}
}